-- author RivalBomb

claasLexion430 = {};

function claasLexion430.prerequisitesPresent(specializations)
	print("......LOADING CLAAS LEXION 430/460 Pack by RIVALBOMB......");
    return SpecializationUtil.hasSpecialization(Motorized, specializations) and SpecializationUtil.hasSpecialization(Overloading, specializations);
end;

function claasLexion430:load(xmlFile)
	if #InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1 == 1 then
		InputBinding.actions[InputBinding.CL_EMPTY_GRAIN].keys1[1] = InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1[1];
	end;
	self.playerCallback = SpecializationUtil.callSpecializationsFunction("playerCallback");

	if self.isClient then
		local node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.unloadingSound#node"));
		self.unloadingSound = Utils.loadSample(xmlFile, {}, "vehicle.unloadingSound", nil, self.baseDirectory, node);
		local node2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.strawChopperSound#node"));
		self.strawChopperSound = Utils.loadSample(xmlFile, {}, "vehicle.strawChopperSound", nil, self.baseDirectory, node2);
		local node3 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.maizeChopperSound#node"));
		self.maizeChopperSound = Utils.loadSample(xmlFile, {}, "vehicle.maizeChopperSound", nil, self.baseDirectory, node3);
	end;
	
	-- Stroh-Haufen
	self.strawBunch = {};
	
	local i = 0;
	while true do
		local key = string.format("vehicle.strawBunches.strawBunch(%d)",i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;

		local nodeWheat = getXMLString(xmlFile, key.. "#indexWheatBarley");
		local nodeRape = getXMLString(xmlFile, key.. "#indexRape");
		local nodeMaize = getXMLString(xmlFile, key.. "#indexMaize");
		local scaleNode = getXMLString(xmlFile, key.. "#scaleNode");
		if nodeWheat == nil or nodeRape == nil or nodeMaize == nil or scaleNode == nil then
			break;
		end;
		local trigger = Utils.indexToObject(self.components, getXMLString(xmlFile, key.. "#trigger"));
		if trigger == nil then
			break;
		end;

		i = i + 1;
		self.strawBunch[i] = {};
		self.strawBunch[i].scaleNode = Utils.indexToObject(self.components, scaleNode);
		self.strawBunch[i].indexWheat = Utils.indexToObject(self.components, nodeWheat);
		self.strawBunch[i].indexRape = Utils.indexToObject(self.components, nodeRape);
		self.strawBunch[i].indexMaize = Utils.indexToObject(self.components, nodeMaize);
		self.strawBunch[i].maxScale = Utils.getNoNil(getXMLFloat(xmlFile, key .. "#maxScale"), 1);
		self.strawBunch[i].resetOnChopperSwitch = Utils.getNoNil(getXMLBool(xmlFile, key .. "#resetOnChopperSwitch"), false);
		self.strawBunch[i].curTrans = 0;
		self.strawBunch[i].trigger = trigger;
		self.strawBunch[i].plIR = false;
		setVisibility(self.strawBunch[i].indexWheat, false);
		setVisibility(self.strawBunch[i].indexRape, false);
		setVisibility(self.strawBunch[i].indexMaize, false);
		
		addTrigger( trigger, "playerCallback", self );
	end;	
	self.threshingStartPS = {};
	Utils.loadParticleSystem(xmlFile, self.threshingStartPS, "vehicle.threshingStartParticleSystem", self.components, false, nil, self.baseDirectory);
	self.threshingStartPSoffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.threshingStartParticleSystem#startOffset"),0.1)*1000;
	self.threshingStartPStimer = 0;
	
	self.interiorLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.interiorLight#index"));
	self.interiorLightActive = false;
end;
function claasLexion430:delete()
	if self.unloadingSound ~= nil then
		Utils.deleteSample(self.unloadingSound);
	end;
	if self.strawChopperSound ~= nil then
		Utils.deleteSample(self.strawChopperSound);
	end;
	if self.maizeChopperSound ~= nil then
		Utils.deleteSample(self.maizeChopperSound);
	end;
	for i,j in pairs(self.strawBunch) do
		if j.trigger ~= nil then
			removeTrigger(j.trigger);
		end;
	end;
	Utils.deleteParticleSystem(self.threshingStartPS);
end;
function claasLexion430:readStream(streamId, connection)
	for i=1, #self.strawBunch do
		self.strawBunch[i].curTrans = streamReadFloat32(streamId);
	end;
end;
function claasLexion430:writeStream(streamId, connection)
	for i=1, #self.strawBunch do
		streamWriteFloat32(streamId, self.strawBunch[i].curTrans);
	end;
end;
function claasLexion430:mouseEvent(posX, posY, isDown, isUp, button)
end;
function claasLexion430:keyEvent(unicode, sym, modifier, isDown)
end;
function claasLexion430:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		for i,j in pairs(self.strawBunch) do
			local s = Utils.getNoNil( getXMLFloat(xmlFile, key..string.format("#strawBunchScale%d",i) ), 0);
			self.strawBunch[i].curTrans = s;
		end;
	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function claasLexion430:getSaveAttributesAndNodes(nodeIdent)
    local attributes = '';
	for i,j in pairs(self.strawBunch) do
		attributes = attributes .. string.format('strawBunchScale%d="%s" ', i, tostring(j.curTrans));
	end;
    return attributes, nil;
end;
function claasLexion430:update(dt)
	if self.isEntered and self:getIsActiveForInput(false) then
		if self.camIndex == 2 and InputBinding.hasEvent(InputBinding.resetCamRotCL430) then
			self.cameras[2].rotX = -25*(math.pi/180);
			self.cameras[2].rotY = math.pi;
			self.cameras[2].rotZ = 0;
		end;
	end;
    if self:getIsActive() then
        if self.isClient and self:getIsActiveForInput(false) then
            if InputBinding.hasEvent(InputBinding.CL_EMPTY_GRAIN) then
                if self.hasPipe then
                    local nextState = self.targetPipeState+1;
                    if nextState > self.numPipeStates then
                        nextState = 1;
                    end;
                    if self:getIsPipeStateChangeAllowed(nextState) then
                        self:setPipeState(nextState);
                    elseif nextState ~= 1 and self:getIsPipeStateChangeAllowed(1) then
                        -- also try to close the pipe if other states are not allowed
                        self:setPipeState(1);
                    end
                else
                    self:setOverloadingActive(not self.isOverloadingActive);
                end;
            end;
			if self:canToggleLight() then
				if InputBinding.hasEvent(InputBinding.SWITCH_INT_LIGHT) then
					self.interiorLightActive = not self.interiorLightActive;
					playSample(g_currentMission.toggleLightsSound, 1, 1.0, 0);
					setVisibility(self.interiorLight, self.interiorLightActive);
				end;
			end;
        end;

		for i=1, #self.strawBunch do
			if self.chopperPSenabled or self.strawPSenabled then
				if self.strawBunch[i].resetOnChopperSwitch then
					if not self.isStrawEnabled then
						self.strawBunch[i].curTrans = math.min(self.strawBunch[i].curTrans+0.000005, self.strawBunch[i].maxScale);
					end;
				else
					self.strawBunch[i].curTrans = math.min(self.strawBunch[i].curTrans+0.000005, self.strawBunch[i].maxScale);
				end;
			end;
			if self.isStrawEnabled and self.strawBunch[i].resetOnChopperSwitch then
				self.strawBunch[i].curTrans = 0;
			end;
			local filltypee = self.currentFillType;
			if self.currentFilltype == Fillable.FILLTYPE_UNKNOWN then
				filltypee = FruitUtils.fruitTypeToFillType[self.lastValidInputFruitType];
			end;
			if filltypee == Fillable.FILLTYPE_WHEAT or filltypee == Fillable.FILLTYPE_BARLEY then
				setVisibility(self.strawBunch[i].indexWheat, true);
			else
				setVisibility(self.strawBunch[i].indexWheat, false);
			end;
			if filltypee == Fillable.FILLTYPE_RAPE then
				setVisibility(self.strawBunch[i].indexRape, true);
			else
				setVisibility(self.strawBunch[i].indexRape, false);
			end;
			if filltypee == Fillable.FILLTYPE_MAIZE then
				setVisibility(self.strawBunch[i].indexMaize, true);
			else
				setVisibility(self.strawBunch[i].indexMaize, false);
			end;
		end;
	else
		self.interiorLightActive = false;
		setVisibility(self.interiorLight, false);
	end;
	local plIR = false;
	local id = 0;
	for i,j in pairs(self.strawBunch) do
		if j.plIR then
			id = i;
			plIR = true;
			break;
		end;
	end;
	if plIR then
		g_currentMission:addHelpButtonText( g_i18n:getText("CLEAR_STRAW_BUNCH"), InputBinding.CLEAR_STRAW_BUNCH);
		if InputBinding.hasEvent(InputBinding.CLEAR_STRAW_BUNCH) then
			self.strawBunch[id].curTrans = 0;
		end;
	end;
	for i=1, #self.strawBunch do
		setScale(self.strawBunch[i].scaleNode, self.strawBunch[i].curTrans, self.strawBunch[i].curTrans, math.min(self.strawBunch[i].curTrans*2, self.strawBunch[i].maxScale));
	end;
	if self.isClient and self.unloadingSound.sound3D ~= nil then
		if self.pipeIsUnloading then
			if not self.unloadingSoundPlaying then
				setVisibility(self.unloadingSound.sound3D, true);
				self.unloadingSoundPlaying = true;
			end;
		else
			if self.unloadingSoundPlaying then
				setVisibility(self.unloadingSound.sound3D, false);
				self.unloadingSoundPlaying = false;
			end;
		end;
	end;
	if self.isClient and self.strawChopperSound.sound3D ~= nil and self.maizeChopperSound.sound3D ~= nil then
		if self.chopperPSenabled then
			if self.cameras[self.camIndex].isInside and self.isEntered then
				setVisibility(self.strawChopperSound.sound3D, false);
				setVisibility(self.maizeChopperSound.sound3D, false);
			else
				if self.lastValidFillType == Fillable.FILLTYPE_MAIZE then
					setVisibility(self.maizeChopperSound.sound3D, true);
				else
					setVisibility(self.strawChopperSound.sound3D, true);
				end;
			end;
		else
			setVisibility(self.strawChopperSound.sound3D, false);
			setVisibility(self.maizeChopperSound.sound3D, false);
		end;
	end;
end;
function claasLexion430:updateTick(dt)
	if self:getIsActive() then
		if self.isClient then
			if self.sampleThreshingStart.isPlaying then
				if self.threshingStartPStimer >= self.threshingStartPSoffset then
					Utils.setEmittingState(self.threshingStartPS, true)
				else
					self.threshingStartPStimer = self.threshingStartPStimer+dt;
				end;
			else
				Utils.setEmittingState(self.threshingStartPS, false);
				self.threshingStartPStimer = 0;
			end;
		end;
	end;
end;
function claasLexion430:draw()
	if self:getIsActiveForInput(false) then
		g_currentMission:addHelpButtonText(g_i18n:getText("SWITCH_INT_LIGHT"), InputBinding.SWITCH_INT_LIGHT);
	end;
end;
function claasLexion430:onEnter()
	if self.isClient then
		if #InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1 == 1 and InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1[1] == InputBinding.actions[InputBinding.CL_EMPTY_GRAIN].keys1[1] then
			table.insert(InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1, 1, Input.KEY_lshift);
			--InputBinding.preprocessInputActions();
		end;
	end;
end;
function claasLexion430:onLeave()
	if self.isClient then
		if #InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1 == 2 and InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1[1] == Input.KEY_lshift then
			table.remove(InputBinding.actions[InputBinding.EMPTY_GRAIN].keys1, 1, Input.KEY_lshift);
			--InputBinding.preprocessInputActions();
		end;
	end;
end;
function claasLexion430:playerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	local id = 0;
	for i,j in pairs(self.strawBunch) do
		if j.trigger == triggerId then
			id = i;
			break;
		end;
	end;
	
	if id ~= 0 then
		if onEnter and g_currentMission.controlPlayer and g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
			self.strawBunch[id].plIR = true;
		elseif onLeave then
			self.strawBunch[id].plIR = false;
		end;
	end
	
end;