--
-- lexionGui
-- Specialization for a additional GUI
--
-- @author:		Michi77
-- @date:		24/12/15
-- @version:	v1.0
-- @history:	v1.0 - initial version
--
-- free for non-commercial usage
-- note: rafftnixGUI needed!!

lexionGui = {};

function lexionGui.prerequisitesPresent(specializations)
    return RafftnixGUI ~= nil;
end;

function lexionGui:load(xmlFile)
	self.gui = RafftnixGUI:new(nil, false, false);
	self.addBackupCam = SpecializationUtil.callSpecializationsFunction("addBackupCam");
	self.removeBackupCam = SpecializationUtil.callSpecializationsFunction("removeBackupCam");
	self.cameraButton1Callback = SpecializationUtil.callSpecializationsFunction("cameraButton1Callback");
	self.cameraButton2Callback = SpecializationUtil.callSpecializationsFunction("cameraButton2Callback");
	self.cameraButton3Callback = SpecializationUtil.callSpecializationsFunction("cameraButton3Callback");
	self.cameraButton4Callback = SpecializationUtil.callSpecializationsFunction("cameraButton4Callback");
	self.cameraButton5Callback = SpecializationUtil.callSpecializationsFunction("cameraButton5Callback");
	self.cameraButtonPipeCallback = SpecializationUtil.callSpecializationsFunction("cameraButtonPipeCallback");
	self.cameraNotAvailable = SpecializationUtil.callSpecializationsFunction("cameraNotAvailable");
	self.cameraButtonOffCallback = SpecializationUtil.callSpecializationsFunction("cameraButtonOffCallback");

	self.blackBackgroundHeight = 0.033;
	self.blackBackground = self.gui:createImage(self.gui.baseElement, 0.8265, 0.1785, 0.174, self.blackBackgroundHeight, "hud/hud_block.dds");
	
	self.backupCams = {};
	self.currentCam = 0;
	self.maxBackupCams = 5;
	-- local posX = getXMLFloat(xmlFile, "vehicle.lexionGui.position#x");
	-- local posY = getXMLFloat(xmlFile, "vehicle.lexionGui.position#y");
	-- local width = getXMLFloat(xmlFile, "vehicle.lexionGui.position#width");
	-- local height = getXMLFloat(xmlFile, "vehicle.lexionGui.position#height");
	local i = 0;
	while true do		
		local key = string.format("vehicle.backupCams.cam(%d)#index", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		local cam = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.backupCams.cam(%d)#index", i)));
		-- local camOverlay = self.gui:createRenderOverlay(self.gui.baseElement, posX, posY+height, width, height, cam, 512, 512, getScreenAspectRatio());
		local camOverlay = self.gui:createRenderOverlay(self.gui.baseElement, 0.831, 0.21, 0.166, 0.166, cam, 512, 512, getScreenAspectRatio());

		self:addBackupCam(cam, camOverlay);
		i = i + 1;
	end;

	self.cameraNotAvailableText = self.gui:createText(self.gui.baseElement, 0.841, 0.2*1.5, 0.166, 0.166, g_i18n:getText("CAM_NOT_AVAILABLE"), 0.015, {1,1,1,1});
	self.cameraNotAvailableText.visibility = false;
	self:cameraButtonOffCallback();
	self.currentCam = 0;
	
	self.gui.cameraButton1 = self.gui:createImgButton(self.gui.baseElement, 0.83, 		   0.18, 0.025, 0.025, "hud/panel_button1_diffuse.dds", self.cameraButton1Callback, self);
	self.gui.cameraButton2 = self.gui:createImgButton(self.gui.baseElement, 0.83+1*0.0265, 0.18, 0.025, 0.025, "hud/panel_button2_diffuse.dds", self.cameraButton2Callback, self);
	self.gui.cameraButton3 = self.gui:createImgButton(self.gui.baseElement, 0.83+2*0.0265, 0.18, 0.025, 0.025, "hud/panel_button3_diffuse.dds", self.cameraButton3Callback, self);
	self.gui.cameraButton4 = self.gui:createImgButton(self.gui.baseElement, 0.83+3*0.0265, 0.18, 0.025, 0.025, "hud/panel_button4_diffuse.dds", self.cameraButton4Callback, self);
	self.gui.cameraButton5 = self.gui:createImgButton(self.gui.baseElement, 0.83+4*0.0265, 0.18, 0.025, 0.025, "hud/panel_button5_diffuse.dds", self.cameraButton5Callback, self);
	self.gui.cameraButtonOff = self.gui:createImgButton(self.gui.baseElement, 0.83+5*0.0265, 0.18, 0.025, 0.025, "hud/panel_button_onoff_diffuse.dds", self.cameraButtonOffCallback, self);
	self.gui.cameraButtonPipe = self.gui:createImgButton(self.gui.baseElement, 0.83+0.155, 0.18, 0.02, 0.02, "hud/panel_buttonPipe_diffuse.dds", self.cameraButtonPipeCallback, self);

	self.isOpened = false;
	self.lastCam = 0;
	self.camChanged = false;
	self.doCamSwitch = 1; -- 1: Switch to pipe cam and turn cam on (if it was off); 2: Switch to pipe cam and don't turn on if off; 3: no switching
end;

function lexionGui:delete()
end;

function lexionGui:addBackupCam(cam, overlay)
	if table.getn(self.backupCams) < self.maxBackupCams then
		self.backupCams[#self.backupCams + 1] = {};
		self.backupCams[#self.backupCams].index = cam;
		if overlay ~= nil then
			self.backupCams[#self.backupCams].overlay = overlay;
		else
			local camOverlay = self.gui:createRenderOverlay(self.gui.baseElement, 0.831, 0.21, 0.166, 0.166, cam, 512, 512, getScreenAspectRatio());
			self.backupCams[#self.backupCams].overlay = camOverlay;
			self.backupCams[#self.backupCams].overlay.visibility = false;
		end;
		--print("Cam Added: "..cam);
	else
		print("cam maximum reached");
	end;
end;

function lexionGui:removeBackupCam(cam)
	for i=1, #self.backupCams do
		if self.backupCams[i].index == cam then
			self.backupCams[i].overlay.visibility = false;
			table.remove(self.backupCams, i);
			break;
		end;
	end;
	if self.currentCam == 4 or self.currentCam == 5 then
		self:cameraNotAvailable(true);
	end;
end;

function lexionGui:mouseEvent(posX, posY, isDown, isUp, button)
end;

function lexionGui:keyEvent(unicode, sym, modifier, isDown)
end;

function lexionGui:update(dt)
	if self.isEntered then
		self.isOpened = not self.isOpened;
		self.gui.mouseActive = self.isMouseActive;
		if g_currentMission.showHudEnv == false then
			self.gui:close();
		else
			self.gui:open();
		end;
		if self.doCamSwitch ~= 3 then
			if self.currentPipeState == 2 or self.targetPipeState == 2 then
				if not self.camChanged then
					self.lastCam = self.currentCam;
					if (self.doCamSwitch == 1) or (self.doCamSwitch == 2 and self.currentCam ~= 0) then
						self:cameraButtonOffCallback();
						if self.backupCams[2] ~= nil then
							self.backupCams[2].overlay.visibility = true;
							self.currentCam = 2;
						end;
						self.blackBackground.height = self.blackBackgroundHeight * 6.2;
					end;
					self.camChanged = true;
				end;
			else
				if self.camChanged then
					self:cameraButtonOffCallback();
					if self.backupCams[self.lastCam] ~= nil then
						self.backupCams[self.lastCam].overlay.visibility = true;
						self.blackBackground.height = self.blackBackgroundHeight * 6.2;
						self.currentCam = self.lastCam;
					end;
					self.camChanged = false;
				end;
			end;
		end;
	else
		self.gui:close();
	end;
end;

function lexionGui:draw()
	renderText(0.83+0.165, 0.175, 0.015,tostring(self.doCamSwitch));
end;

function lexionGui:cameraNotAvailable(state)
	self.cameraNotAvailableText.visibility = state;
end;
function lexionGui:cameraButton1Callback()
	self:cameraButtonOffCallback();
	if self.backupCams[1] ~= nil then self.backupCams[1].overlay.visibility = true; self.currentCam = 1; else self:cameraNotAvailable(true); end;
	self.blackBackground.height = self.blackBackgroundHeight * 6.2;
end;
function lexionGui:cameraButton2Callback()
	self:cameraButtonOffCallback();
	if self.backupCams[2] ~= nil then self.backupCams[2].overlay.visibility = true; self.currentCam = 2; else self:cameraNotAvailable(true); end;
	self.blackBackground.height = self.blackBackgroundHeight * 6.2;
end;
function lexionGui:cameraButton3Callback()
	self:cameraButtonOffCallback();
	if self.backupCams[3] ~= nil then self.backupCams[3].overlay.visibility = true; self.currentCam = 3; else self:cameraNotAvailable(true); end;
	self.blackBackground.height = self.blackBackgroundHeight * 6.2;
end;
function lexionGui:cameraButton4Callback()
	self:cameraButtonOffCallback();
	if self.backupCams[4] ~= nil then self.backupCams[4].overlay.visibility = true; self.currentCam = 4; else self:cameraNotAvailable(true); end;
	self.blackBackground.height = self.blackBackgroundHeight * 6.2;
end;
function lexionGui:cameraButton5Callback()
	self:cameraButtonOffCallback();
	if self.backupCams[5] ~= nil then self.backupCams[5].overlay.visibility = true; self.currentCam = 5; else self:cameraNotAvailable(true); end;
	self.blackBackground.height = self.blackBackgroundHeight * 6.2;
end;
function lexionGui:cameraButtonPipeCallback()
	self.doCamSwitch = self.doCamSwitch+1;
	if self.doCamSwitch >= 4 then
		self.doCamSwitch = 1;
	end;
end;
function lexionGui:cameraButtonOffCallback()
	self:cameraNotAvailable(false);
	if self.backupCams[1] ~= nil then self.backupCams[1].overlay.visibility = false; end;
	if self.backupCams[2] ~= nil then self.backupCams[2].overlay.visibility = false; end;
	if self.backupCams[3] ~= nil then self.backupCams[3].overlay.visibility = false; end;
	if self.backupCams[4] ~= nil then self.backupCams[4].overlay.visibility = false; end;
	if self.backupCams[5] ~= nil then self.backupCams[5].overlay.visibility = false; end;
	self.blackBackground.height = self.blackBackgroundHeight;
	self.currentCam = 0;
end;